@@ -1,14 +1,9 @@
 package Tapper::PRC;
-# git description: v4.1.1-11-g75f1086
+# git description: v4.1.2-40-g3b6a62a
 
-BEGIN {
-  $Tapper::PRC::AUTHORITY = 'cpan:TAPPER';
-}
-{
-  $Tapper::PRC::VERSION = '4.1.2';
-}
+our $AUTHORITY = 'cpan:TAPPER';
 # ABSTRACT: Tapper - Program run control for test program automation
-
+$Tapper::PRC::VERSION = '5.0.0';
 use strict;
 use warnings;
 
@@ -41,9 +36,10 @@ sub mcp_error
 1;
 
 __END__
+
 =pod
 
-=encoding utf-8
+=encoding UTF-8
 
 =head1 NAME
 
@@ -64,17 +60,26 @@ Log an error and exit.
 
 @return never returns
 
-=head1 AUTHOR
+=head1 AUTHORS
+
+=over 4
+
+=item *
 
 AMD OSRC Tapper Team <tapper@amd64.org>
 
+=item *
+
+Tapper Team <tapper-ops@amazon.com>
+
+=back
+
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2012 by Advanced Micro Devices, Inc..
+This software is Copyright (c) 2016 by Advanced Micro Devices, Inc..
 
 This is free software, licensed under:
 
   The (two-clause) FreeBSD License
 
 =cut
-
