@@ -1,11 +1,6 @@
 package Tapper::PRC::Testcontrol;
-BEGIN {
-  $Tapper::PRC::Testcontrol::AUTHORITY = 'cpan:TAPPER';
-}
-{
-  $Tapper::PRC::Testcontrol::VERSION = '4.1.2';
-}
-
+our $AUTHORITY = 'cpan:TAPPER';
+$Tapper::PRC::Testcontrol::VERSION = '5.0.0';
 use 5.010;
 use warnings;
 use strict;
@@ -18,7 +13,7 @@ use YAML 'LoadFile';
 use File::Basename 'dirname';
 use English '-no_match_vars';
 use IO::Handle;
-use File::Basename 'basename';
+use File::Basename qw/basename dirname/;
 
 use Tapper::Remote::Config;
 # ABSTRACT: Control running test programs
@@ -40,7 +35,6 @@ sub capture_handler_tap
 }
 
 
-
 sub send_output
 {
         my ($self, $captured_output, $testprogram) = @_;
@@ -54,48 +48,137 @@ sub send_output
         $captured_output =~ s/^(1\.\.\d+\n)/$1$headerlines/m;
 
         my ($error, $message) = $self->tap_report_away($captured_output);
+
         return $message if $error;
         return 0;
 
 }
 
 
+sub send_attachements {
+
+    my ( $self ) = @_;
+
+    my ( $b_error, $s_message ) = $self->tap_report_away(
+          "TAP version 13\n"
+        . "1..1\n"
+        . "# Tapper-suite-name: PRC" . ( $self->cfg->{guest_number} || 0 ) . "-Attachments\n"
+        . "# Tapper-machine-name: " . $self->cfg->{hostname} . "\n"
+        . "# Tapper-reportgroup-testrun: " . $self->cfg->{test_run} . "\n"
+        . "ok - Test attachments\n"
+    );
+
+    return ( 1, $s_message ) if $b_error;
+
+    $self->upload_files( $s_message );
+
+    return ( 0, q## );
+
+}
+
+
+sub upload_files
+{
+
+    my ( $or_self, $i_reportid ) = @_;
+
+    my $s_host = $or_self->cfg->{report_server};
+    my $i_port = $or_self->cfg->{report_api_port};
+    my $s_path = $ENV{TAPPER_OUTPUT_PATH};
+
+    return 0 unless -d $s_path;
+
+    my @a_files = `find $s_path -type f`;
+
+    $or_self->log->debug( @a_files );
+
+    foreach my $s_file( @a_files ) {
+
+        chomp $s_file;
+
+        my $s_reportfile =  $s_file;
+           $s_reportfile =~ s|^$s_path/*||;
+           $s_reportfile =~ s|^./||;
+           $s_reportfile =~ s|[^A-Za-z0-9_-]|_|g;
+
+        my $or_server = IO::Socket::INET->new(
+            PeerAddr => $s_host,
+            PeerPort => $i_port,
+        );
+
+        return "Cannot open remote receiver $s_host:$i_port" if not $or_server;
+
+        open( my $fh_file, "<", $s_file ) or do{$or_self->log->warn("Can't open $s_file:$!"); $or_server->close();next;};
+        $or_server->print("#! upload $i_reportid $s_reportfile plain\n");
+        while ( my $line = <$fh_file> ) {
+                $or_server->print($line);
+        }
+        close($fh_file);
+        $or_server->close();
+
+    }
+
+    return 0;
+
+}
+
+
+sub get_appendix {
+        my($self, $output)  = @_;
+        my $appendix = '';
+        if (-e "$output.stdout" or -e "$output.stderr") {
+                my $basename = basename($output);
+                my $dirname  = dirname ($output);
+                my @files  = <$dirname/$basename-*.stdout>;
+
+                no warnings 'uninitialized';
+                my @appendizes = sort map { my ($append) = m/(\d+)\D*$/; $append} @files;
+                $appendix = sprintf("-%03d",shift(@appendizes) + 1);
+        }
+        return $appendix;
+}
+
 
 sub testprogram_execute
 {
         my ($self, $test_program) = @_;
 
-        my $program  = $test_program->{program};
-        my $chdir    = $test_program->{chdir};
+        my $program  =  $test_program->{program};
+        my $chdir    =  $test_program->{chdir};
         my $progpath =  $self->cfg->{paths}{testprog_path};
         my $output   =  $program;
-        $output      =~ s|[^A-Za-z0-9_-]|_|g;
-        $output      =  $test_program->{out_dir}.$output;
+           $output   =~ s|[^A-Za-z0-9_-]|_|g;
+           $output   =  $test_program->{out_dir}.$output;
 
 
-        # make relative paths absolute
-        $program=$progpath.$program if $program !~ m(^/);
-
-        # try to catch non executables early
-        return("tried to execute $program which does not exist") unless -e $program;
-
-
-        if (not -x $program) {
-                system ("chmod", "ugo+x", $program);
-                return("tried to execute $program which is not an execuable and can not set exec flag") if not -x $program;
+        if ($program !~ m(^/)) {
+                $ENV{PATH} = "$progpath:$ENV{PATH}";
+                $program = qx(which $program);
+                chomp $program;
         }
 
-        return("tried to execute $program which is a directory") if -d $program;
-        return("tried to execute $program which is a special file (FIFO, socket, device, ..)") unless -f $program or -l $program;
+        # try to catch non executables early
+        if (-e $program) {
+                if (not -x $program) {
+                        system ("chmod", "ugo+x", $program);
+                        return("tried to execute $program which is not an execuable and can not set exec flag") if not -x $program;
+                }
+
+                return("tried to execute $program which is a directory") if -d $program;
+                return("tried to execute $program which is a special file (FIFO, socket, device, ..)") unless -f $program or -l $program;
+        }
 
         foreach my $file (@{$test_program->{upload_before} || [] }) {
+
                 my $target_name =~ s|[^A-Za-z0-9_-]|_|g;
-                $target_name = $test_program->{out_dir}.'/before/'.$target_name;
+                   $target_name = $test_program->{out_dir}.'/before/'.$target_name;
                 File::Copy::copy($file, $target_name);
+
         }
 
         $self->log->info("Try to execute test suite $program");
 
+        my $appendix = $self->get_appendix($output);
         pipe (my $read, my $write);
         return ("Can't open pipe:$!") if not (defined $read and defined $write);
 
@@ -104,13 +187,14 @@ sub testprogram_execute
 
         if ($pid == 0) {        # hello child
                 close $read;
+
                 %ENV = (%ENV, %{$test_program->{environment} || {} });
-                open (STDOUT, ">", "$output.stdout") or syswrite($write, "Can't open output file $output.stdout: $!"),exit 1;
-                open (STDERR, ">", "$output.stderr") or syswrite($write, "Can't open output file $output.stderr: $!"),exit 1;
+                open (STDOUT, ">", "$output$appendix.stdout") or syswrite($write, "Can't open output file $output$appendix.stdout: $!"),exit 1;
+                open (STDERR, ">", "$output$appendix.stderr") or syswrite($write, "Can't open output file $output$appendix.stderr: $!"),exit 1;
                 if ($chdir) {
                         if (-d $chdir) {
                                 chdir $chdir;
-                        } elsif ($chdir == "AUTO" and $program =~ m,^/, ) {
+                        } elsif ($chdir eq "AUTO" and $program =~ m,^/, ) {
                                 chdir dirname($program);
                         }
                 }
@@ -118,23 +202,26 @@ sub testprogram_execute
                 close $write;
                 exit -1;
         } else {
+
                 # hello parent
                 close $write;
-                my $killed;
-                # (XXX) better create a process group an kill this
-                local $SIG{ALRM}=sub {
-                                      $killed = 1;
-                                      kill (15, $pid);
 
-                                      # allow testprogram to react on SIGTERM
-                                      my $grace_period = $ENV{HARNESS_ACTIVE} ? 1 : 60; # wait less during test
-                                      while ($grace_period and (kill 0, $pid)) {
-                                              waitpid($pid,0);
-                                              sleep 1;
-                                              $grace_period--;
-                                      }
-                                      kill (9, $pid);
-                                     };
+                # (XXX) better create a process group an kill this
+                my $killed;
+                local $SIG{ALRM} = sub {
+                    $killed = 1;
+                    kill (15, $pid);
+
+                    # allow testprogram to react on SIGTERM
+                    my $grace_period = $ENV{HARNESS_ACTIVE} ? 1 : 60; # wait less during test
+                    while ( $grace_period > 0 and (kill 0, $pid) ) {
+                        sleep 1;
+                        $grace_period--;
+                    }
+
+                    kill (9, $pid);
+                };
+
                 alarm ($test_program->{timeout} || 0);
                 waitpid($pid,0);
                 my $retval = $?;
@@ -147,12 +234,19 @@ sub testprogram_execute
                 }
                 if ($test_program->{capture}) {
                         my $captured_output;
-                        given($test_program->{capture}) {
-                                when ('tap') { eval { $captured_output = $self->capture_handler_tap("$output.stdout")}; return $@ if $@;};
-                                default      { return "Can not handle captured output, unknown capture type '$test_program->{capture}'. Valid types are (tap)"};
+                        if ( $test_program->{capture} eq 'tap' ) {
+                                eval { $captured_output = $self->capture_handler_tap("$output$appendix.stdout")};
+                                return $@ if $@;
                         }
-                        my $error_msg =  $self->send_output($captured_output, $test_program);
-                        return $error_msg if $error_msg;
+                        elsif ( $test_program->{capture} eq 'tap-stderr' ) {
+                            eval { $captured_output = $self->capture_handler_tap("$output$appendix.stderr")};
+                            return $@ if $@;
+                        }
+                        else               {
+                            return "Can not handle captured output, unknown capture type '$test_program->{capture}'. Valid types are (tap)";
+                        }
+                        my ( $b_error, $error_msg ) =  $self->send_output($captured_output, $test_program);
+                        return $error_msg if $b_error;
                 }
 
                 return "Killed $program after $test_program->{timeout} seconds" if $killed;
@@ -166,7 +260,6 @@ sub testprogram_execute
 }
 
 
-
 sub guest_start
 {
         my ($self) = @_;
@@ -245,7 +338,7 @@ sub guest_start
         return 0;
 }
 
-
+        
 sub create_log
 {
         my ($self) = @_;
@@ -272,8 +365,6 @@ sub create_log
 }
 
 
-
-
 sub nfs_mount
 {
         my ($self) = @_;
@@ -294,6 +385,7 @@ sub nfs_mount
 sub control_testprogram
 {
         my ($self) = @_;
+
         $ENV{TAPPER_TESTRUN}         = $self->cfg->{test_run};
         $ENV{TAPPER_SERVER}          = $self->cfg->{mcp_server};
         $ENV{TAPPER_REPORT_SERVER}   = $self->cfg->{report_server};
@@ -304,24 +396,20 @@ sub control_testprogram
         $ENV{TAPPER_MAX_REBOOT}      = $self->cfg->{max_reboot} if $self->cfg->{max_reboot};
         $ENV{TAPPER_GUEST_NUMBER}    = $self->cfg->{guest_number} || 0;
         $ENV{TAPPER_SYNC_FILE}       = $self->cfg->{syncfile} if $self->cfg->{syncfile};
-        $ENV{CRITICALITY}            = $self->cfg->{criticality} //  4;  # provide criticality for autoreport test scripts (4 == max)
+        $ENV{TAPPER_SYNC_PATH}       = $self->cfg->{paths}{sync_path}; # if -d ($self->cfg->{paths}{sync_path} || '');
         if ($self->{cfg}->{testplan}) {
                 $ENV{TAPPER_TESTPLAN_ID}   = $self->cfg->{testplan}{id};
                 $ENV{TAPPER_TESTPLAN_PATH} = $self->cfg->{testplan}{path};
         }
 
-
-
         my $test_run         = $self->cfg->{test_run};
         my $out_dir          = $self->cfg->{paths}{output_dir}."/$test_run/test/";
         my @testprogram_list;
-        @testprogram_list    = @{$self->cfg->{testprogram_list}} if $self->cfg->{testprogram_list};
-
+           @testprogram_list = @{$self->cfg->{testprogram_list}} if $self->cfg->{testprogram_list};
 
         # prepend outdir with guest number if we are in virtualisation guest
         $out_dir.="guest-".$self->{cfg}->{guest_number}."/" if $self->{cfg}->{guest_number};
 
-
         my $error = $self->makedir($out_dir);
 
         # can't create output directory. Make
@@ -330,7 +418,7 @@ sub control_testprogram
                 $out_dir = tempdir( CLEANUP => 1 );
         }
 
-        $ENV{TAPPER_OUTPUT_PATH}=$out_dir;
+        $ENV{TAPPER_OUTPUT_PATH} = $out_dir;
 
         if ($self->cfg->{test_program}) {
                 my $argv;
@@ -385,7 +473,6 @@ sub control_testprogram
 }
 
 
-
 sub get_peers_from_file
 {
         my ($self, $file) = @_;
@@ -405,7 +492,6 @@ sub get_peers_from_file
 }
 
 
-
 sub wait_for_sync
 {
         my ($self, $syncfile) = @_;
@@ -481,13 +567,13 @@ sub send_keep_alive_loop
 }
 
 
-
 sub run
 {
         my ($self) = @_;
-        my $retval;
+
         my $producer = Tapper::Remote::Config->new();
-        my $config = $producer->get_local_data("test-prc0");
+        my $config   = $producer->get_local_data("test-prc0");
+
         $self->cfg($config);
         $self->cfg->{reboot_counter} = 0 if not defined($self->cfg->{reboot_counter});
 
@@ -495,44 +581,43 @@ sub run
                 $self->log_to_file('testing');
         }
 
-        if ($config->{times}{keep_alive_timeout}) {
-                $SIG{CHLD} = 'IGNORE';
-                my $pid = fork();
-                if ($pid == 0) {
-                        $self->send_keep_alive_loop($config->{times}{keep_alive_timeout});
-                        exit;
-                } else {
-                        $config->{keep_alive_child} = $pid;
-                }
-        }
-
         # ignore error
         $self->log_and_exec('ntpdate -s gwo');
 
         if ($config->{prc_nfs_server}) {
-                $retval = $self->nfs_mount();
-                $self->log->warn($retval) if $retval;
+                if ( my $retval = $self->nfs_mount() ) {
+                        $self->log->warn($retval);
+                }
         }
 
-        $self->log->logdie($retval) if $retval = $self->create_log();
+        if ( my $retval = $self->create_log() ) {
+                $self->log->logdie($retval);
+        }
 
         if ($config->{scenario_id}) {
                 my $syncfile = $config->{paths}{sync_path}."/".$config->{scenario_id}."/syncfile";
-                $self->cfg->{syncfile} = $syncfile;
+                if (-e $syncfile) {
+                        $self->cfg->{syncfile} = $syncfile;
 
-                $retval = $self->wait_for_sync($syncfile);
-                $self->log->logdie("Can not sync - $retval") if $retval;
+                        if ( my $retval = $self->wait_for_sync($syncfile) ) {
+                                $self->log->logdie("Can not sync - $retval");
+                        }
+                }
         }
 
         if ($self->{cfg}->{guest_count}) {
-
-                $retval = $self->guest_start();
-                $self->log->error($retval) if $retval;
+                if ( my $retval = $self->guest_start() ) {
+                        $self->log->error($retval);
+                }
         }
 
-        $retval = $self->mcp_inform({state => 'start-testing'}) if not $self->cfg->{reboot_counter};
+        if ( not $self->cfg->{reboot_counter} ) {
+                $self->mcp_inform({state => 'start-testing'});
+        }
 
-        $retval = $self->control_testprogram() if $self->cfg->{test_program} or $self->cfg->{testprogram_list};
+        if ( $self->cfg->{test_program} or $self->cfg->{testprogram_list} ) {
+                $self->control_testprogram();
+        }
 
         if ($self->cfg->{max_reboot}) {
                 $self->mcp_inform({state => 'reboot', count => $self->cfg->{reboot_counter}, max_reboot => $self->cfg->{max_reboot}});
@@ -545,25 +630,27 @@ sub run
 
         }
 
-
-        # no longer send keepalive
-        if ($config->{keep_alive_child}) {
-                kill 15, $config->{keep_alive_child};
-                sleep 2;
-                kill 9, $config->{keep_alive_child};
-        }
         sleep 1; # make sure last end-testing can't overtake last end-testprogram (Yes, this did happen)
-        $retval = $self->mcp_inform({state => 'end-testing'});
 
+        # send attachment report
+        my ( $b_error, $s_error_msg ) = $self->send_attachements();
+        if ( $b_error ) {
+                $self->log->error( $s_error_msg );
+        }
+
+        $self->mcp_inform({state => 'end-testing'});
+
+        return 1;
 
 }
 
 1;
 
 __END__
+
 =pod
 
-=encoding utf-8
+=encoding UTF-8
 
 =head1 NAME
 
@@ -591,6 +678,35 @@ Send the captured TAP output to the report receiver.
 @return success - 0
 @return error   - error string
 
+=head2 send_output
+
+Send the a attachment to the report receiver and add attachements.
+
+@return success - 0
+@return error   - error string
+
+=head2 upload_files
+
+Upload files written in one stage of the testrun to report framework.
+
+@param int - report id
+@param int - testrun id
+
+@return success - 0
+@return error   - error string
+
+=head2 get_appendix
+
+For testprogram with the same name the output file names will be
+identical. To prevent this, we append a serial number. This function
+calculates this appendix and returns the next one to use. If no such
+serial is needed because no output file of the given name exists yet the
+empty string is returned.
+
+@param string  - name of the output file without appendix
+
+@return string - string to append to output file name to make it unique
+
 =head2 testprogram_execute
 
 Execute one testprogram. Handle all error conditions.
@@ -667,17 +783,26 @@ Send keepalive messages to MCP in an endless loop.
 
 Main function of Program Run Control.
 
-=head1 AUTHOR
+=head1 AUTHORS
+
+=over 4
+
+=item *
 
 AMD OSRC Tapper Team <tapper@amd64.org>
 
+=item *
+
+Tapper Team <tapper-ops@amazon.com>
+
+=back
+
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2012 by Advanced Micro Devices, Inc..
+This software is Copyright (c) 2016 by Advanced Micro Devices, Inc..
 
 This is free software, licensed under:
 
   The (two-clause) FreeBSD License
 
 =cut
-
